########### SETTINGS TO ADJUST ################################################
# 2024-04-19
# https://www.drassal.net/wp

# script to easily generate self signed SSL certificates

# this script adds the ability to generate certificates with fine tuned valid
# times (down to the second), adjust $CRT_VALID_SECONDS to set it

# this requires knowledge of the current timezone to be able to calculate
# the begin and end valid times correctly, don't forget to change it

# for PST timezone, -8 hours
TIMEZONE_OFFSET_SECONDS=-28800

# for JST timezone, +9 hours
#TIMEZONE_OFFSET_SECONDS=32400

# parameters for signing request
CSR_COUNTRY="US"
CSR_STATE="CA"
CSR_LOCALITY="Los Angeles"
CSR_ORGANIZATION=""
CSR_ORGANIZATIONUNIT=""
###############################################################################

COMMON_NAME=$1
CRT_VALID_DAYS=$2
FILEPATH_CERT=$3

FILEPATH_ROOT=${PWD}
FILEPATH_CA="${FILEPATH_ROOT}/ca"
FILEPATH_CERTIFICATES="${FILEPATH_ROOT}/certificates"
FILEPATH_PERSISTENT_STORAGE="${FILEPATH_ROOT}/persistent_storage"

FILEPATH_CERT_OPENSSL_CONF=${FILEPATH_ROOT}/openssl.cnf
FILEPATH_CERT_CA_KEY="${FILEPATH_CA}/ca.key"
FILEPATH_CERT_CA_CRT="${FILEPATH_CA}/ca.crt"

if ! test -f ${FILEPATH_CERT_CA_KEY}; then
  echo "${FILEPATH_CERT_CA_KEY} does not exist, run create_ca.sh first"
  exit
fi

if ! test -f ${FILEPATH_CERT_CA_CRT}; then
  echo "${FILEPATH_CERT_CA_CRT} does not exist, run create_ca.sh first"
  exit
fi

if ! [ -n "$COMMON_NAME" ]
then
  echo "COMMON_NAME (first command line parameter) is empty"
  echo "example for domain \"www.mydomain.net\" valid for 90 days"
  echo "./create_cert_seconds.sh www.mydomain.net 90"
  exit
fi

if ! [ -n "$CRT_VALID_DAYS" ]
then
  CRT_VALID_DAYS=90
fi

if [ -n "$FILEPATH_CERT" ]
then
  FILEPATH_CERT="${FILEPATH_CERTIFICATES}/${FILEPATH_CERT}"
else
  FILEPATH_CERT="${FILEPATH_CERTIFICATES}/${COMMON_NAME}"
fi

SECONDS_PER_DAY=86400
CRT_VALID_SECONDS=60

NOW=$(date +"%Y-%m-%d %H:%M:%S")
NOW_YMD=$(date -jf "%Y-%m-%d %H:%M:%S" "${NOW}" +"%Y%m%d")
NOW_YMDHMS=$(date -jf "%Y-%m-%d %H:%M:%S" "${NOW}" +"%Y%m%d%H%M%S")

CRT_STARTDATE_EPOCH_LOCAL=$(date -jf "%Y-%m-%d %H:%M:%S" "${NOW}" +"%s")
CRT_ENDDATE_EPOCH_LOCAL=`expr $(date -jf "%Y-%m-%d %H:%M:%S" "${NOW}" +"%s") + $CRT_VALID_SECONDS`
CRT_STARTDATE_EPOCH=`expr ${CRT_STARTDATE_EPOCH_LOCAL} - $TIMEZONE_OFFSET_SECONDS`
CRT_ENDDATE_EPOCH=`expr ${CRT_ENDDATE_EPOCH_LOCAL} - $TIMEZONE_OFFSET_SECONDS`
CRT_STARTDATE=$(date -jf "%s" ${CRT_STARTDATE_EPOCH} +"%y%m%d%H%M%S")
CRT_ENDDATE=$(date -jf "%s" ${CRT_ENDDATE_EPOCH} +"%y%m%d%H%M%S")

CRT_STARTDATE_YMDHMS=$(date -jf "%s" ${CRT_STARTDATE_EPOCH_LOCAL} +"%Y-%m-%d-%H-%M-%S")
CRT_ENDDATE_YMDHMS=$(date -jf "%s" ${CRT_ENDDATE_EPOCH_LOCAL} +"%Y-%m-%d-%H-%M-%S")

# Signed certificates storage
FILEPATH_CERT_CA_DB_CERTS=${FILEPATH_PERSISTENT_STORAGE}/ca.db.certs
# Index of signed certificates
FILEPATH_CERT_CA_DB_INDEX=${FILEPATH_PERSISTENT_STORAGE}/ca.db.index
FILEPATH_CERT_CA_DB_INDEX_ATTR=${FILEPATH_PERSISTENT_STORAGE}/ca.db.index.attr
# Next (sequential) serial number
FILEPATH_CERT_CA_DB_SERIAL=${FILEPATH_PERSISTENT_STORAGE}/ca.db.serial

#create necessary support files for signing (if not exist)
if ! test -f ${FILEPATH_CERT_CA_DB_CERTS}; then
  mkdir -p ${FILEPATH_CERT_CA_DB_CERTS}
fi
if ! test -f ${FILEPATH_CERT_CA_DB_INDEX}; then
  touch ${FILEPATH_CERT_CA_DB_INDEX}
fi
if ! [ -f ${FILEPATH_CERT_CA_DB_INDEX_ATTR} ]; then
  echo "unique_subject = no" > ${FILEPATH_CERT_CA_DB_INDEX_ATTR}
fi
if ! [ -f ${FILEPATH_CERT_CA_DB_SERIAL} ]; then
  echo 01 > ${FILEPATH_CERT_CA_DB_SERIAL}
fi

FILEPATH_CERT_KEY=${FILEPATH_CERT}/${COMMON_NAME}_${NOW_YMDHMS}.key
FILEPATH_CERT_CSR=${FILEPATH_CERT}/${COMMON_NAME}_${NOW_YMDHMS}.csr
FILEPATH_CERT_CRT=${FILEPATH_CERT}/${COMMON_NAME}_${NOW_YMDHMS}.crt

echo ${FILEPATH_CERT}
if ! test -f ${FILEPATH_CERT}; then
  mkdir -p ${FILEPATH_CERT}
fi

#generate private key
openssl genrsa \
        -out $FILEPATH_CERT_KEY \
        2048

#generate signing request
CSR_COMMONNAME=${COMMON_NAME}
CSR_SUBJECT="/C=${CSR_COUNTRY}/ST=${CSR_STATE}/L=${CSR_LOCALITY}/O=${CSR_ORGANIZATION}/OU=${CSR_ORGANIZATIONUNIT}/CN=${CSR_COMMONNAME}/"
openssl req \
        -sha256 \
        -new \
        -out ${FILEPATH_CERT_CSR} \
        -key ${FILEPATH_CERT_KEY} \
        -subj "${CSR_SUBJECT}"

#check the CSR request
openssl req \
        -text \
        -noout \
        -verify \
        -in ${FILEPATH_CERT_CSR}

#sign request and create certificate (simple method)
#openssl x509 \
#        -req \
#        -days ${CRT_VALID_DAYS} \
#        -CA    ${FILEPATH_CERT_CA_CRT} \
#        -CAkey ${FILEPATH_CERT_CA_KEY} \
#        -CAcreateserial \
#        -in  ${FILEPATH_CERT_CSR} \
#        -out ${FILEPATH_CERT_CRT}
#sign request and create certificate (can fine tune valid date/time)

openssl ca \
        -md sha256 \
        -config ${FILEPATH_CERT_OPENSSL_CONF} \
        -policy generic_policy \
        -out ${FILEPATH_CERT_CRT} \
        -startdate ${CRT_STARTDATE}Z \
        -enddate ${CRT_ENDDATE}Z \
        -keyfile ${FILEPATH_CERT_CA_KEY} \
        -cert ${FILEPATH_CERT_CA_CRT} \
        -notext \
        -infiles ${FILEPATH_CERT_CSR}    

#check signed SSL
openssl x509 \
        -in ${FILEPATH_CERT_CRT} \
        -text \
        -noout
